# Copyright 2002 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Invoke rdiff utility to make signatures, deltas, or patch"""

import os, librsync
import Globals, log, static, TempFile, rpath, hash


def get_signature(rp, blocksize = None):
	"""Take signature of rpin file and return in file object"""
	if not blocksize: blocksize = find_blocksize(rp.getsize())
	log.Log("Getting signature of %s with blocksize %s" %
			(rp.get_indexpath(), blocksize), 7)
	return librsync.SigFile(rp.open("rb"), blocksize)

def find_blocksize(file_len):
	"""Return a reasonable block size to use on files of length file_len

	If the block size is too big, deltas will be bigger than is
	necessary.  If the block size is too small, making deltas and
	patching can take a really long time.

	"""
	if file_len < 4096: return 64 # set minimum of 64 bytes
	else: # Use square root, rounding to nearest 16
		return long(pow(file_len, 0.5)/16)*16

def get_delta_sigfileobj(sig_fileobj, rp_new):
	"""Like get_delta but signature is in a file object"""
	log.Log("Getting delta of %s with signature stream" % (rp_new.path,), 7)
	return librsync.DeltaFile(sig_fileobj, rp_new.open("rb"))

def get_delta_sigrp(rp_signature, rp_new):
	"""Take signature rp and new rp, return delta file object"""
	log.Log("Getting delta of %s with signature %s" %
			(rp_new.path, rp_signature.get_indexpath()), 7)
	return librsync.DeltaFile(rp_signature.open("rb"), rp_new.open("rb"))

def get_delta_sigrp_hash(rp_signature, rp_new):
	"""Like above but also calculate hash of new as close() value"""
	log.Log("Getting delta (with hash) of %s with signature %s" %
			(rp_new.path, rp_signature.get_indexpath()), 7)
	return librsync.DeltaFile(rp_signature.open("rb"),
							  hash.FileWrapper(rp_new.open("rb")))
	

def write_delta(basis, new, delta, compress = None):
	"""Write rdiff delta which brings basis to new"""
	log.Log("Writing delta %s from %s -> %s" %
			(basis.path, new.path, delta.path), 7)
	deltafile = librsync.DeltaFile(get_signature(basis), new.open("rb"))
	delta.write_from_fileobj(deltafile, compress)

def write_patched_fp(basis_fp, delta_fp, out_fp):
	"""Write patched file to out_fp given input fps.  Closes input files"""
	rpath.copyfileobj(librsync.PatchedFile(basis_fp, delta_fp), out_fp)
	assert not basis_fp.close() and not delta_fp.close()

def write_via_tempfile(fp, rp):
	"""Write fileobj fp to rp by writing to tempfile and renaming"""
	tf = TempFile.new(rp)
	retval = tf.write_from_fileobj(fp)
	rpath.rename(tf, rp)
	return retval

def patch_local(rp_basis, rp_delta, outrp = None, delta_compressed = None):
	"""Patch routine that must be run locally, writes to outrp

	This should be run local to rp_basis because it needs to be a real
	file (librsync may need to seek around in it).  If outrp is None,
	patch rp_basis instead.

	The return value is the close value of the delta, so it can be
	used to produce hashes.

	"""
	assert rp_basis.conn is Globals.local_connection
	if delta_compressed: deltafile = rp_delta.open("rb", 1)
	else: deltafile = rp_delta.open("rb")
	patchfile = librsync.PatchedFile(rp_basis.open("rb"), deltafile)
	if outrp: return outrp.write_from_fileobj(patchfile)
	else: return write_via_tempfile(patchfile, rp_basis)

def copy_local(rpin, rpout, rpnew = None):
	"""Write rpnew == rpin using rpout as basis.  rpout and rpnew local"""
	assert rpout.conn is Globals.local_connection
	deltafile = rpin.conn.librsync.DeltaFile(get_signature(rpout),
											 rpin.open("rb"))
	patched_file = librsync.PatchedFile(rpout.open("rb"), deltafile)

	if rpnew: rpnew.write_from_fileobj(patched_file)
	else: write_via_tempfile(patched_file, rpout)
	





# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Define some lazy data structures and functions acting on them"""

from __future__ import generators
import os, stat, types
import static


class Iter:
	"""Hold static methods for the manipulation of lazy iterators"""

	def filter(predicate, iterator):
		"""Like filter in a lazy functional programming language"""
		for i in iterator:
			if predicate(i): yield i

	def map(function, iterator):
		"""Like map in a lazy functional programming language"""
		for i in iterator: yield function(i)

	def foreach(function, iterator):
		"""Run function on each element in iterator"""
		for i in iterator: function(i)

	def cat(*iters):
		"""Lazily concatenate iterators"""
		for iter in iters:
			for i in iter: yield i

	def cat2(iter_of_iters):
		"""Lazily concatenate iterators, iterated by big iterator"""
		for iter in iter_of_iters:
			for i in iter: yield i

	def empty(iter):
		"""True if iterator has length 0"""
		for i in iter: return None
		return 1

	def equal(iter1, iter2, verbose = None, operator = lambda x, y: x == y):
		"""True if iterator 1 has same elements as iterator 2

		Use equality operator, or == if it is unspecified.

		"""
		for i1 in iter1:
			try: i2 = iter2.next()
			except StopIteration:
				if verbose: print "End when i1 = %s" % (i1,)
				return None
			if not operator(i1, i2):
				if verbose: print "%s not equal to %s" % (i1, i2)
				return None
		try: i2 = iter2.next()
		except StopIteration: return 1
		if verbose: print "End when i2 = %s" % (i2,)
		return None

	def Or(iter):
		"""True if any element in iterator is true.  Short circuiting"""
		i = None
		for i in iter:
			if i: return i
		return i

	def And(iter):
		"""True if all elements in iterator are true.  Short circuiting"""
		i = 1
		for i in iter:
			if not i: return i
		return i

	def len(iter):
		"""Return length of iterator"""
		i = 0
		while 1:
			try: iter.next()
			except StopIteration: return i
			i = i+1

	def foldr(f, default, iter):
		"""foldr the "fundamental list recursion operator"?"""
		try: next = iter.next()
		except StopIteration: return default
		return f(next, Iter.foldr(f, default, iter))

	def foldl(f, default, iter):
		"""the fundamental list iteration operator.."""
		while 1:
			try: next = iter.next()
			except StopIteration: return default
			default = f(default, next)

	def multiplex(iter, num_of_forks, final_func = None, closing_func = None):
		"""Split a single iterater into a number of streams

		The return val will be a list with length num_of_forks, each
		of which will be an iterator like iter.  final_func is the
		function that will be called on each element in iter just as
		it is being removed from the buffer.  closing_func is called
		when all the streams are finished.

		"""
		if num_of_forks == 2 and not final_func and not closing_func:
			im2 = IterMultiplex2(iter)
			return (im2.yielda(), im2.yieldb())
		if not final_func: final_func = lambda i: None
		if not closing_func: closing_func = lambda: None

		# buffer is a list of elements that some iterators need and others
		# don't
		buffer = []

		# buffer[forkposition[i]] is the next element yieled by iterator
		# i.  If it is -1, yield from the original iter
		starting_forkposition = [-1] * num_of_forks
		forkposition = starting_forkposition[:]
		called_closing_func = [None]

		def get_next(fork_num):
			"""Return the next element requested by fork_num"""
			if forkposition[fork_num] == -1:
				try:  buffer.insert(0, iter.next())
				except StopIteration:
					# call closing_func if necessary
					if (forkposition == starting_forkposition and
						not called_closing_func[0]):
						closing_func()
						called_closing_func[0] = None
					raise StopIteration
				for i in range(num_of_forks): forkposition[i] += 1

			return_val = buffer[forkposition[fork_num]]
			forkposition[fork_num] -= 1

			blen = len(buffer)
			if not (blen-1) in forkposition:
				# Last position in buffer no longer needed
				assert forkposition[fork_num] == blen-2
				final_func(buffer[blen-1])
				del buffer[blen-1]
			return return_val

		def make_iterator(fork_num):
			while(1): yield get_next(fork_num)

		return tuple(map(make_iterator, range(num_of_forks)))

static.MakeStatic(Iter)


class IterMultiplex2:
	"""Multiplex an iterator into 2 parts

	This is a special optimized case of the Iter.multiplex function,
	used when there is no closing_func or final_func, and we only want
	to split it into 2.  By profiling, this is a time sensitive class.

	"""
	def __init__(self, iter):
		self.a_leading_by = 0 # How many places a is ahead of b
		self.buffer = []
		self.iter = iter

	def yielda(self):
		"""Return first iterator"""
		buf, iter = self.buffer, self.iter
		while(1):
			if self.a_leading_by >= 0: # a is in front, add new element
				elem = iter.next() # exception will be passed
				buf.append(elem)
			else: elem = buf.pop(0) # b is in front, subtract an element
			self.a_leading_by += 1
			yield elem

	def yieldb(self):
		"""Return second iterator"""
		buf, iter = self.buffer, self.iter
		while(1):
			if self.a_leading_by <= 0: # b is in front, add new element
				elem = iter.next() # exception will be passed
				buf.append(elem)
			else: elem = buf.pop(0) # a is in front, subtract an element
			self.a_leading_by -= 1
			yield elem



# Copyright 2002 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Provides a high-level interface to some librsync functions

This is a python wrapper around the lower-level _librsync module,
which is written in C.  The goal was to use C as little as possible...

"""

import types, array
import _librsync

blocksize = _librsync.RS_JOB_BLOCKSIZE

class librsyncError(Exception):
	"""Signifies error in internal librsync processing (bad signature, etc.)

	underlying _librsync.librsyncError's are regenerated using this
	class because the C-created exceptions are by default
	unPickleable.  There is probably a way to fix this in _librsync,
	but this scheme was easier.

	"""
	pass


class LikeFile:
	"""File-like object used by SigFile, DeltaFile, and PatchFile"""
	mode = "rb"

	# This will be replaced in subclasses by an object with
	# appropriate cycle() method
	maker = None

	def __init__(self, infile, need_seek = None):
		"""LikeFile initializer - zero buffers, set eofs off"""
		self.check_file(infile, need_seek)
		self.infile = infile
		self.closed = self.infile_closed = None
		self.inbuf = ""
		self.outbuf = array.array('c')
		self.eof = self.infile_eof = None

	def check_file(self, file, need_seek = None):
		"""Raise type error if file doesn't have necessary attributes"""
		if not hasattr(file, "read"):
			raise TypeError("Basis file must have a read() method")
		if not hasattr(file, "close"):
			raise TypeError("Basis file must have a close() method")
		if need_seek and not hasattr(file, "seek"):
			raise TypeError("Basis file must have a seek() method")

	def read(self, length = -1):
		"""Build up self.outbuf, return first length bytes"""
		if length == -1:
			while not self.eof: self._add_to_outbuf_once()
			real_len = len(self.outbuf)
		else:
			while not self.eof and len(self.outbuf) < length:
				self._add_to_outbuf_once()
			real_len = min(length, len(self.outbuf))
			
		return_val = self.outbuf[:real_len].tostring()
		del self.outbuf[:real_len]
		return return_val

	def _add_to_outbuf_once(self):
		"""Add one cycle's worth of output to self.outbuf"""
		if not self.infile_eof: self._add_to_inbuf()
		try: self.eof, len_inbuf_read, cycle_out = self.maker.cycle(self.inbuf)
		except _librsync.librsyncError, e: raise librsyncError(str(e))
		self.inbuf = self.inbuf[len_inbuf_read:]
		self.outbuf.fromstring(cycle_out)

	def _add_to_inbuf(self):
		"""Make sure len(self.inbuf) >= blocksize"""
		assert not self.infile_eof
		while len(self.inbuf) < blocksize:
			new_in = self.infile.read(blocksize)
 			if not new_in:
				self.infile_eof = 1
				self.infile_closeval = self.infile.close()
				self.infile_closed = 1
				break
			self.inbuf += new_in

	def close(self):
		"""Close infile and pass on infile close value"""
		self.closed = 1
		if self.infile_closed: return self.infile_closeval
		else: return self.infile.close()


class SigFile(LikeFile):
	"""File-like object which incrementally generates a librsync signature"""
	def __init__(self, infile, blocksize = _librsync.RS_DEFAULT_BLOCK_LEN):
		"""SigFile initializer - takes basis file

		basis file only needs to have read() and close() methods.  It
		will be closed when we come to the end of the signature.

		"""
		LikeFile.__init__(self, infile)
		try: self.maker = _librsync.new_sigmaker(blocksize)
		except _librsync.librsyncError, e: raise librsyncError(str(e))

class DeltaFile(LikeFile):
	"""File-like object which incrementally generates a librsync delta"""
	def __init__(self, signature, new_file):
		"""DeltaFile initializer - call with signature and new file

		Signature can either be a string or a file with read() and
		close() methods.  New_file also only needs to have read() and
		close() methods.  It will be closed when self is closed.

		"""
		LikeFile.__init__(self, new_file)
		if type(signature) is types.StringType: sig_string = signature
		else:
			self.check_file(signature)
			sig_string = signature.read()
			assert not signature.close()
		try: self.maker = _librsync.new_deltamaker(sig_string)
		except _librsync.librsyncError, e: raise librsyncError(str(e))


class PatchedFile(LikeFile):
	"""File-like object which applies a librsync delta incrementally"""
	def __init__(self, basis_file, delta_file):
		"""PatchedFile initializer - call with basis delta

		Here basis_file must be a true Python file, because we may
		need to seek() around in it a lot, and this is done in C.
		delta_file only needs read() and close() methods.

		"""
		LikeFile.__init__(self, delta_file)
		if hasattr(basis_file, 'file'):
			basis_file = basis_file.file
		if type(basis_file) is not types.FileType:
			raise TypeError("basis_file must be a (true) file")
		try: self.maker = _librsync.new_patchmaker(basis_file)
		except _librsync.librsyncError, e: raise librsyncError(str(e))		


class SigGenerator:
	"""Calculate signature.

	Input and output is same as SigFile, but the interface is like md5
	module, not filelike object

	"""
	def __init__(self, blocksize = _librsync.RS_DEFAULT_BLOCK_LEN):
		"""Return new signature instance"""
		try: self.sig_maker = _librsync.new_sigmaker(blocksize)
		except _librsync.librsyncError, e: raise librsyncError(str(e))
		self.gotsig = None
		self.buffer = ""
		self.sig_string = ""

	def update(self, buf):
		"""Add buf to data that signature will be calculated over"""
		if self.gotsig:
			raise librsyncError("SigGenerator already provided signature")
		self.buffer += buf
		while len(self.buffer) >= blocksize:
			if self.process_buffer():
				raise librsyncError("Premature EOF received from sig_maker")

	def process_buffer(self):
		"""Run self.buffer through sig_maker, add to self.sig_string"""
		try: eof, len_buf_read, cycle_out = self.sig_maker.cycle(self.buffer)
		except _librsync.librsyncError, e: raise librsyncError(str(e))
		self.buffer = self.buffer[len_buf_read:]
		self.sig_string += cycle_out
		return eof

	def getsig(self):
		"""Return signature over given data"""
		while not self.process_buffer(): pass # keep running until eof
		return self.sig_string




# Copyright 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Handle long filenames

rdiff-backup sometimes wants to write filenames longer than allowed by
the destination directory.  This can happen in 3 ways:

1)  Because the destination directory has a low maximum length limit.
2)  When the source directory has a filename close to the limit, so
    that its increments would be above the limit.
3)  When quoting is enabled, so that even the mirror filenames are too
    long.

When rdiff-backup would otherwise write a file whose name is too long,
instead it either skips the operation altogether (for non-regular
files), or writes the data to a unique file in the
rdiff-backup-data/long-filename directory.  This file will have an
arbitrary basename, but if it's an increment the suffix will be the
same.  The name will be recorded in the mirror_metadata so we can find
it later.

"""

import types, errno
import log, Globals, restore, rpath, FilenameMapping, regress

long_name_dir = "long_filename_data"
rootrp = None

def get_long_rp(base = None):
	"""Return an rpath in long name directory with given base"""
	global rootrp
	if not rootrp:
		rootrp = Globals.rbdir.append(long_name_dir)
		if not rootrp.lstat(): rootrp.mkdir()
	if base: return rootrp.append(base)
	else: return rootrp


# ------------------------------------------------------------------
# These functions used mainly for backing up

# integer number of next free prefix.  Names will be created from
# integers consecutively like '1', '2', and so on.
free_name_counter = None

# Filename which holds the next available free name in it
counter_filename = "next_free"

def get_next_free():
	"""Return next free filename available in the long filename directory"""
	global free_name_counter
	def scan_next_free():
		"""Return value of free_name_counter by listing long filename dir"""
		log.Log("Setting next free from long filenames dir", 5)
		cur_high = 0
		for filename in get_long_rp().listdir():
			try: i = int(filename.split('.')[0])
			except ValueError: continue
			if i > cur_high: cur_high = i
		return cur_high + 1

	def read_next_free():
		"""Return next int free by reading the next_free file, or None"""
		rp = get_long_rp(counter_filename)
		if not rp.lstat(): return None
		return int(rp.get_data())

	def write_next_free(i):
		"""Write value i into the counter file"""
		rp = get_long_rp(counter_filename)
		if rp.lstat(): rp.delete()
		rp.write_string(str(free_name_counter))
		rp.fsync_with_dir()

	if not free_name_counter: free_name_counter = read_next_free()
	if not free_name_counter: free_name_counter = scan_next_free()
	filename = str(free_name_counter)
	rp = get_long_rp(filename)
	assert not rp.lstat(), "Unexpected file at %s found" % (rp.path,)
	free_name_counter += 1
	write_next_free(free_name_counter)
	return filename


def check_new_index(base, index, make_dirs = 0):
	"""Return new rpath with given index, or None if that is too long

	If make_dir is True, make any parent directories to assure that
	file is really too long, and not just in directories that don't exist.

	"""
	def wrap_call(func, *args):
		try: result = func(*args)
		except EnvironmentError, exc:
			if (errno.errorcode.has_key(exc[0]) and
				errno.errorcode[exc[0]] == 'ENAMETOOLONG'):
				return None
			if (exc[1] == "The filename or extension is too long"):
				return None
			raise
		return result

	def make_parent(rp):
		parent = rp.get_parent_rp()
		if parent.lstat(): return 1
		parent.makedirs()
		return 2

	rp = wrap_call(base.new_index, index)
	if not make_dirs or not rp or rp.lstat(): return rp

	parent_result = wrap_call(make_parent, rp)
	if not parent_result: return None
	elif parent_result == 1: return rp
	else: return wrap_call(base.new_index, index)

def get_mirror_rp(mirror_base, mirror_rorp):
	"""Get the mirror_rp for reading a regular file

	This will just be in the mirror_base, unless rorp has an alt
	mirror name specified.  Use new_rorp, unless it is None or empty,
	and mirror_rorp exists.

	"""
	if mirror_rorp.has_alt_mirror_name():
		return get_long_rp(mirror_rorp.get_alt_mirror_name())
	else:
		rp = check_new_index(mirror_base, mirror_rorp.index)
		if rp: return rp
		else: return mirror_base.new_index_empty(index)

def get_mirror_inc_rps(rorp_pair, mirror_root, inc_root = None):
	"""Get (mirror_rp, inc_rp) pair, possibly making new longname base

	To test inc_rp, pad incbase with 50 random (non-quoted) characters
	and see if that raises an error.

	"""
	if not inc_root: # make fake inc_root if not available
		inc_root = mirror_root.append_path('rdiff-backup-data/increments')

	def mir_triple_old(old_rorp):
		"""Return (mirror_rp, alt_mirror, alt_inc) from old_rorp"""
		if old_rorp.has_alt_mirror_name():
			alt_mirror = old_rorp.get_alt_mirror_name()
			return (get_long_rp(alt_mirror), alt_mirror, None)
		else:
			mirror_rp = mirror_root.new_index(old_rorp.index)
			if old_rorp.has_alt_inc_name():
				return (mirror_rp, None, old_rorp.get_alt_inc_name())
			else: return (mirror_rp, None, None)

	def mir_triple_new(new_rorp):
		"""Return (mirror_rp, alt_mirror, None) from new_rorp"""
		mirror_rp = check_new_index(mirror_root, new_rorp.index)
		if mirror_rp: return (mirror_rp, None, None)
		alt_mirror = get_next_free()
		return (get_long_rp(alt_mirror), alt_mirror, None)

	def update_rorp(new_rorp, alt_mirror, alt_inc):
		"""Update new_rorp with alternate mirror/inc information"""
		if not new_rorp or not new_rorp.lstat(): return
		if alt_mirror: new_rorp.set_alt_mirror_name(alt_mirror)
		elif alt_inc: new_rorp.set_alt_inc_name(alt_inc)

	def find_inc_pair(index, mirror_rp, alt_mirror, alt_inc):
		"""Return (alt_inc, inc_rp) pair"""
		if alt_mirror: return (None, mirror_rp)
		elif alt_inc: return (alt_inc, get_long_rp(alt_inc))
		elif not index: return (None, inc_root)

		trial_inc_index = index[:-1] + (index[-1] + ('a'*50),)
		if check_new_index(inc_root, trial_inc_index, make_dirs = 1):
			return (None, inc_root.new_index(index))
		alt_inc = get_next_free()
		return (alt_inc, get_long_rp(alt_inc))
		
	(new_rorp, old_rorp) = rorp_pair
	if old_rorp and old_rorp.lstat():
		mirror_rp, alt_mirror, alt_inc = mir_triple_old(old_rorp)
		index = old_rorp.index
	else:
		assert new_rorp and new_rorp.lstat(), (old_rorp, new_rorp)
		mirror_rp, alt_mirror, alt_inc = mir_triple_new(new_rorp)
		index = new_rorp.index

	alt_inc, inc_rp = find_inc_pair(index, mirror_rp, alt_mirror, alt_inc)
	update_rorp(new_rorp, alt_mirror, alt_inc)
	return mirror_rp, inc_rp


# ------------------------------------------------------------------
# The following section is for restoring

# This holds a dictionary {incbase: inclist}.  The keys are increment
# bases like '1' or '23', and the values are lists containing the
# associated increments.
restore_inc_cache = None

def set_restore_cache():
	"""Initialize restore_inc_cache based on long filename dir"""
	global restore_inc_cache
	restore_inc_cache = {}
	root_rf = restore.RestoreFile(get_long_rp(), get_long_rp(), [])
	for incbase_rp, inclist in root_rf.yield_inc_complexes(get_long_rp()):
		restore_inc_cache[incbase_rp.index[-1]] = inclist

def get_inclist(inc_base_name):
	if not restore_inc_cache: set_restore_cache()
	try: return restore_inc_cache[inc_base_name]
	except KeyError: return []

def update_rf(rf, rorp, mirror_root):
	"""Return new or updated restorefile based on alt name info in rorp"""
	def update_incs(rf, inc_base):
		"""Swap inclist in rf with those with base inc_base and return"""
		log.Log("Restoring with increment base %s for file %s" %
				(inc_base, rorp.get_indexpath()), 6)		
		rf.inc_rp = get_long_rp(inc_base)
		rf.inc_list = get_inclist(inc_base)
		rf.set_relevant_incs()

	def update_existing_rf(rf, rorp):
		"""Update rf based on rorp, don't make new one"""
		if rorp.has_alt_mirror_name():
			inc_name = rorp.get_alt_mirror_name()
			rf.mirror_rp = get_long_rp(mirror_name)
		elif rorp.has_alt_inc_name(): inc_name = rorp.get_alt_inc_name()
		else: inc_name = None

		if inc_name: update_incs(rf, inc_name)

	def make_new_rf(rorp, mirror_root):
		"""Make a new rf when long name info is available"""
		if rorp.has_alt_mirror_name():
			inc_name = rorp.get_alt_mirror_name()
			mirror_rp = get_long_rp(inc_name)
		else:
			mirror_rp = mirror_root.new_index(rorp.index)
			if rorp.has_alt_inc_name(): inc_name = rorp.get_alt_inc_name()
			else: return restore.RestoreFile(mirror_rp, None, [])

		rf = restore.RestoreFile(mirror_rp, None, [])
		update_incs(rf, inc_name)
		return rf
		
	if not rorp: return rf
	if rf and not rorp.has_alt_mirror_name() and not rorp.has_alt_inc_name():
		return rf # Most common case
	if rf:
		update_existing_rf(rf, rorp)
		return rf
	else: return make_new_rf(rorp, mirror_root)

def update_regressfile(rf, rorp, mirror_root):
	"""Like update_rf except return a regress file object"""
	rf = update_rf(rf, rorp, mirror_root)
	if isinstance(rf, regress.RegressFile): return rf
	return regress.RegressFile(rf.mirror_rp, rf.inc_rp, rf.inc_list)



# Copyright 2003 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Store and retrieve extended attributes and access control lists

Not all file systems will have EAs and ACLs, but if they do, store
this information in separate files in the rdiff-backup-data directory,
called extended_attributes.<time>.snapshot and
access_control_lists.<time>.snapshot.

"""

from __future__ import generators
import base64, errno, re
try: import posix1e
except ImportError: pass
import static, Globals, eas_acls, connection, metadata, rorpiter, log, C, \
	   rpath, user_group

# When an ACL gets dropped, put name in dropped_acl_names.  This is
# only used so that only the first dropped ACL for any given name
# triggers a warning.
dropped_acl_names = {}

class ExtendedAttributes:
	"""Hold a file's extended attribute information"""
	def __init__(self, index, attr_dict = None):
		"""Initialize EA object with no attributes"""
		self.index = index
		if attr_dict is None: self.attr_dict = {}
		else: self.attr_dict = attr_dict

	def __eq__(self, ea):
		"""Equal if all attributes are equal"""
		assert isinstance(ea, ExtendedAttributes)
		return ea.attr_dict == self.attr_dict
	def __ne__(self, ea): return not self.__eq__(ea)

	def get_indexpath(self): return self.index and '/'.join(self.index) or '.'

	def read_from_rp(self, rp):
		"""Set the extended attributes from an rpath"""
		try:
			attr_list = rp.conn.xattr.listxattr(rp.path, rp.issym())
		except IOError, exc:
			if exc[0] in (errno.EOPNOTSUPP, errno.EPERM, errno.ETXTBSY):
				return # if not supported, consider empty
			if exc[0] in (errno.EACCES, errno.ENOENT, errno.ELOOP):
				log.Log("Warning: listattr(%s): %s" % (repr(rp.path), exc), 4)
				return
			raise
		for attr in attr_list:
			if attr.startswith('system.'):
				# Do not preserve system extended attributes
				continue
			if not rp.isdir() and attr == 'com.apple.ResourceFork':
				# Resource Fork handled elsewhere, except for directories
				continue
			try:
				self.attr_dict[attr] = \
					rp.conn.xattr.getxattr(rp.path, attr, rp.issym())
			except IOError, exc:
				# File probably modified while reading, just continue
				if exc[0] == errno.ENODATA: continue
				elif exc[0] == errno.ENOENT: break
				# Handle bug in pyxattr < 0.2.2
				elif exc[0] == errno.ERANGE: continue
				else: raise

	def clear_rp(self, rp):
		"""Delete all the extended attributes in rpath"""
		try:
			for name in rp.conn.xattr.listxattr(rp.path, rp.issym()):
				try:
					rp.conn.xattr.removexattr(rp.path, name, rp.issym())
				except IOError, exc:
					# SELinux attributes cannot be removed, and we don't want
					# to bail out or be too noisy at low log levels.
					if exc[0] == errno.EACCES:
						log.Log("Warning: unable to remove xattr %s from %s"
							% (name, repr(rp.path)), 7)
						continue
					else: raise
		except IOError, exc:
			if exc[0] == errno.EOPNOTSUPP or exc[0] == errno.EPERM:
				return # if not supported, consider empty
			elif exc[0] == errno.ENOENT: # path is bad
				log.Log("Warning: unable to clear xattrs on %s: %s" %
						(repr(rp.path), exc), 3)
				return
			else: raise

	def write_to_rp(self, rp):
		"""Write extended attributes to rpath rp"""
		self.clear_rp(rp)
		for (name, value) in self.attr_dict.iteritems():
			try:
				rp.conn.xattr.setxattr(rp.path, name, value, 0, rp.issym())
			except IOError, exc:
				# Mac and Linux attributes have different namespaces, so
				# fail gracefully if can't call setxattr
				if exc[0] in (errno.EOPNOTSUPP, errno.EPERM, errno.EACCES,
						errno.ENOENT, errno.EINVAL):
					log.Log("Warning: unable to write xattr %s to %s"
							% (name, repr(rp.path)), 6)
					continue
				else: raise

	def get(self, name):
		"""Return attribute attached to given name"""
		return self.attr_dict[name]

	def set(self, name, value = ""):
		"""Set given name to given value.  Does not write to disk"""
		self.attr_dict[name] = value

	def delete(self, name):
		"""Delete value associated with given name"""
		del self.attr_dict[name]

	def empty(self):
		"""Return true if no extended attributes are set"""
		return not self.attr_dict

def ea_compare_rps(rp1, rp2):
	"""Return true if rp1 and rp2 have same extended attributes"""
	ea1 = ExtendedAttributes(rp1.index)
	ea1.read_from_rp(rp1)
	ea2 = ExtendedAttributes(rp2.index)
	ea2.read_from_rp(rp2)
	return ea1 == ea2


def EA2Record(ea):
	"""Convert ExtendedAttributes object to text record"""
	str_list = ['# file: %s' % C.acl_quote(ea.get_indexpath())]
	for (name, val) in ea.attr_dict.iteritems():
		if not val: str_list.append(name)
		else:
			encoded_val = base64.encodestring(val).replace('\n', '')
			try:
				str_list.append('%s=0s%s' % (C.acl_quote(name), encoded_val))
			except UnicodeEncodeError:
				log.Log("Warning: unable to store Unicode extended attribute %s"
							% repr(name), 3)
	return '\n'.join(str_list)+'\n'

def Record2EA(record):
	"""Convert text record to ExtendedAttributes object"""
	lines = record.split('\n')
	first = lines.pop(0)
	if not first[:8] == "# file: ":
		raise metadata.ParsingError("Bad record beginning: " + first[:8])
	filename = first[8:]
	if filename == '.': index = ()
	else: index = tuple(C.acl_unquote(filename).split('/'))
	ea = ExtendedAttributes(index)

	for line in lines:
		line = line.strip()
		if not line: continue
		assert line[0] != '#', line
		eq_pos = line.find('=')
		if eq_pos == -1: ea.set(line)
		else:
			name = line[:eq_pos]
			assert line[eq_pos+1:eq_pos+3] == '0s', \
				   "Currently only base64 encoding supported"
			encoded_val = line[eq_pos+3:]
			ea.set(name, base64.decodestring(encoded_val))
	return ea


class EAExtractor(metadata.FlatExtractor):
	"""Iterate ExtendedAttributes objects from the EA information file"""
	record_boundary_regexp = re.compile('(?:\\n|^)(# file: (.*?))\\n')
	record_to_object = staticmethod(Record2EA)
	def filename_to_index(self, filename):
		"""Convert possibly quoted filename to index tuple"""
		if filename == '.': return ()
		else: return tuple(C.acl_unquote(filename).split('/'))

class ExtendedAttributesFile(metadata.FlatFile):
	"""Store/retrieve EAs from extended_attributes file"""
	_prefix = "extended_attributes"
	_extractor = EAExtractor
	_object_to_record = staticmethod(EA2Record)

def join_ea_iter(rorp_iter, ea_iter):
	"""Update a rorp iter by adding the information from ea_iter"""
	for rorp, ea in rorpiter.CollateIterators(rorp_iter, ea_iter):
		assert rorp, "Missing rorp for index %s" % (ea.index,)
		if not ea: ea = ExtendedAttributes(rorp.index)
		rorp.set_ea(ea)
		yield rorp


class AccessControlLists:
	"""Hold a file's access control list information

	Since posix1e.ACL objects cannot be pickled, and because they lack
	user/group name information, store everything in self.entry_list
	and self.default_entry_list.

	"""
	def __init__(self, index, acl_text = None):
		"""Initialize object with index and possibly acl_text"""
		self.index = index
		if acl_text: self.set_from_text(acl_text)
		else: self.entry_list = self.default_entry_list = None

	def set_from_text(self, text):
		"""Set self.entry_list and self.default_entry_list from text"""
		self.entry_list, self.default_entry_list = [], []
		for line in text.split('\n'):
			comment_pos = line.find('#')
			if comment_pos >= 0: line = line[:comment_pos]
			line = line.strip()
			if not line: continue

			if line.startswith('default:'):
				entrytuple = self.text_to_entrytuple(line[8:])
				self.default_entry_list.append(entrytuple)
			else: self.entry_list.append(self.text_to_entrytuple(line))

	def __str__(self):
		"""Return text version of acls"""
		if not self.entry_list: return ""
		slist = map(self.entrytuple_to_text, self.entry_list)
		if self.default_entry_list:
			slist.extend(map(lambda e: "default:" + self.entrytuple_to_text(e),
							 self.default_entry_list))
		return "\n".join(slist)

	def entrytuple_to_text(self, entrytuple):
		"""Return text version of entrytuple, as in getfacl"""
		tagchar, name_pair, perms = entrytuple
		if tagchar == "U": text = 'user::'
		elif tagchar == "u":
			uid, uname = name_pair
			text = 'user:%s:' % (uname or uid)
		elif tagchar == "G":
			text = 'group::'
		elif tagchar == "g":
			gid, gname = name_pair
			text = 'group:%s:' % (gname or gid)
		elif tagchar == "M": text = 'mask::'
		else:
			assert tagchar == "O", tagchar
			text = 'other::'

		permstring = '%s%s%s' % (perms & 4 and 'r' or '-',
								 perms & 2 and 'w' or '-',
								 perms & 1 and 'x' or '-')
		return text+permstring

	def text_to_entrytuple(self, text):
		"""Return entrytuple given text like 'user:foo:r--'

		See the acl_to_list function for entrytuple documentation.

		"""
		typetext, qualifier, permtext = text.split(':')
		if qualifier:
			try: uid = int(qualifier)
			except ValueError: namepair = (None, qualifier)
			else: namepair = (uid, None)

			if typetext == 'user': typechar = "u"
			else:
				assert typetext == 'group', (typetext, text)
				typechar = "g"
		else:
			namepair = None
			if typetext == 'user': typechar = "U"
			elif typetext == 'group': typechar = "G"
			elif typetext == 'mask': typechar = "M"
			else:
				assert typetext == 'other', (typetext, text)
				typechar = "O"

		assert len(permtext) == 3, (permtext, text)
		read, write, execute = permtext
		perms = ((read == 'r') << 2 |
				 (write == 'w') << 1 |
				 (execute == 'x'))
		return (typechar, namepair, perms)

	def cmp_entry_list(self, l1, l2):
		"""True if the lists have same entries.  Assume preordered"""
		if not l1: return not l2
		if not l2 or len(l1) != len(l2): return 0
		for i in range(len(l1)):
			type1, namepair1, perms1 = l1[i]
			type2, namepair2, perms2 = l2[i]
			if type1 != type2 or perms1 != perms2: return 0
			if namepair1 == namepair2: continue
			if not namepair1 or not namepair2: return 0
			(id1, name1), (id2, name2) = namepair1, namepair2
			if name1:
				if name1 == name2: continue
				else: return 0
			if name2: return 0
			if id1 != id2: return 0
		return 1

	def __eq__(self, acl):
		"""Compare self and other access control list

		Basic acl permissions are considered equal to an empty acl
		object.

		"""
		assert isinstance(acl, self.__class__)
		if self.is_basic(): return acl.is_basic()
		return (self.cmp_entry_list(self.entry_list, acl.entry_list) and
				self.cmp_entry_list(self.default_entry_list,
									acl.default_entry_list))
	
	def __ne__(self, acl): return not self.__eq__(acl)
	
	def eq_verbose(self, acl):
		"""Returns same as __eq__ but print explanation if not equal"""
		if not self.cmp_entry_list(self.entry_list, acl.entry_list):
			print "ACL entries for %s compare differently" % (self.index,)
			return 0
		if not self.cmp_entry_list(self.default_entry_list,
								   acl.default_entry_list):
			print "Default ACL entries for %s do not compare" % (self.index,)
			return 0
		return 1

	def get_indexpath(self): return self.index and '/'.join(self.index) or '.'

	def is_basic(self):
		"""True if acl can be reduced to standard unix permissions

		Assume that if they are only three entries, they correspond to
		user, group, and other, and thus don't use any special ACL
		features.

		"""
		if not self.entry_list and not self.default_entry_list: return 1
		assert len(self.entry_list) >= 3, self.entry_list
		return len(self.entry_list) == 3 and not self.default_entry_list

	def read_from_rp(self, rp):
		"""Set self.ACL from an rpath, or None if not supported"""
		self.entry_list, self.default_entry_list = \
						 rp.conn.eas_acls.get_acl_lists_from_rp(rp)

	def write_to_rp(self, rp, map_names = 1):
		"""Write current access control list to RPath rp"""
		rp.conn.eas_acls.set_rp_acl(rp, self.entry_list,
									self.default_entry_list, map_names)


def set_rp_acl(rp, entry_list = None, default_entry_list = None,
			   map_names = 1):
	"""Set given rp with ACL that acl_text defines.  rp should be local"""
	assert rp.conn is Globals.local_connection
	if entry_list: acl = list_to_acl(entry_list, map_names)
	else: acl = posix1e.ACL()

	try:
		acl.applyto(rp.path)
	except IOError, exc:
		if exc[0] == errno.EOPNOTSUPP:
			log.Log("Warning: unable to set ACL on %s: %s" % 
					(repr(rp.path), exc), 4)
			return
		else: raise

	if rp.isdir():
		if default_entry_list:
			def_acl = list_to_acl(default_entry_list, map_names)
		else: def_acl = posix1e.ACL()
		def_acl.applyto(rp.path, posix1e.ACL_TYPE_DEFAULT)

def get_acl_lists_from_rp(rp):
	"""Returns (acl_list, def_acl_list) from an rpath.  Call locally"""
	assert rp.conn is Globals.local_connection
	try: acl = posix1e.ACL(file=rp.path)
	except IOError, exc:
		if exc[0] == errno.EOPNOTSUPP:
			acl = None
		elif exc[0] == errno.ENOENT:
			log.Log("Warning: unable to read ACL from %s: %s"
					% (repr(rp.path), exc), 3)
			acl = None
		else: raise
	if rp.isdir():
		try: def_acl = posix1e.ACL(filedef=rp.path)
		except IOError, exc:
			if exc[0] == errno.EOPNOTSUPP:
				def_acl = None
			elif exc[0] == errno.ENOENT:
				log.Log("Warning: unable to read default ACL from %s: %s"
					% (repr(rp.path), exc), 3)
				def_acl = None
			else: raise
	else: def_acl = None
	return (acl and acl_to_list(acl), def_acl and acl_to_list(def_acl))

def acl_to_list(acl):
	"""Return list representation of posix1e.ACL object

	ACL objects cannot be pickled, so this representation keeps
	the structure while adding that option.  Also we insert the
	username along with the id, because that information will be
	lost when moved to another system.

	The result will be a list of tuples.  Each tuple will have the
	form (acltype, (uid or gid, uname or gname) or None, permissions
	as an int).  acltype is encoded as a single character:

	U - ACL_USER_OBJ
	u - ACL_USER
	G - ACL_GROUP_OBJ
	g - ACL_GROUP
	M - ACL_MASK
	O - ACL_OTHER

	"""
	def acltag_to_char(tag):
		if tag == posix1e.ACL_USER_OBJ: return "U"
		elif tag == posix1e.ACL_USER: return "u"
		elif tag == posix1e.ACL_GROUP_OBJ: return "G"
		elif tag == posix1e.ACL_GROUP: return "g"
		elif tag == posix1e.ACL_MASK: return "M"
		else:
			assert tag == posix1e.ACL_OTHER, tag
			return "O"

	def entry_to_tuple(entry):
		tagchar = acltag_to_char(entry.tag_type)
		if tagchar == "u":
			uid = entry.qualifier
			owner_pair = (uid, user_group.uid2uname(uid))
		elif tagchar == "g":
			gid = entry.qualifier
			owner_pair = (gid, user_group.gid2gname(gid))
		else: owner_pair = None

		perms = (entry.permset.read << 2 | 
				 entry.permset.write << 1 |
				 entry.permset.execute)
		return (tagchar, owner_pair, perms)
	return map(entry_to_tuple, acl)

def list_to_acl(entry_list, map_names = 1):
	"""Return posix1e.ACL object from list representation

	If map_names is true, use user_group to update the names for the
	current system, and drop if not available.  Otherwise just use the
	same id.

	See the acl_to_list function for the format of an acllist.

	"""
	def char_to_acltag(typechar):
		"""Given typechar, query posix1e module for appropriate constant"""
		if typechar == "U": return posix1e.ACL_USER_OBJ
		elif typechar == "u": return posix1e.ACL_USER
		elif typechar == "G": return posix1e.ACL_GROUP_OBJ
		elif typechar == "g": return posix1e.ACL_GROUP
		elif typechar == "M": return posix1e.ACL_MASK
		else:
			assert typechar == "O", typechar
			return posix1e.ACL_OTHER

	def warn_drop(name):
		"""Warn about acl with name getting dropped"""
		global dropped_acl_names
		if Globals.never_drop_acls:
			log.Log.FatalError(
"--never-drop-acls specified but cannot map name\n"
"%s occurring inside an ACL." % (name,))
		if dropped_acl_names.has_key(name): return
		log.Log("Warning: name %s not found on system, dropping ACL entry.\n"
				"Further ACL entries dropped with this name will not "
				"trigger further warnings" % (name,), 2)
		dropped_acl_names[name] = name

	acl = posix1e.ACL()
	for typechar, owner_pair, perms in entry_list:
		id = None
		if owner_pair:
			if map_names:
				if typechar == "u": id = user_group.acl_user_map(*owner_pair)
				else:
					assert typechar == "g", (typechar, owner_pair, perms)
					id = user_group.acl_group_map(*owner_pair)
				if id is None:
					warn_drop(owner_pair[1])
					continue
			else:
				assert owner_pair[0] is not None, (typechar, owner_pair, perms)
				id = owner_pair[0]

		entry = posix1e.Entry(acl)
		entry.tag_type = char_to_acltag(typechar)
		if id is not None: entry.qualifier = id
		entry.permset.read = perms >> 2
		entry.permset.write = perms >> 1 & 1
		entry.permset.execute = perms & 1
	return acl

def acl_compare_rps(rp1, rp2):
	"""Return true if rp1 and rp2 have same acl information"""
	acl1 = AccessControlLists(rp1.index)
	acl1.read_from_rp(rp1)
	acl2 = AccessControlLists(rp2.index)
	acl2.read_from_rp(rp2)
	return acl1 == acl2


def ACL2Record(acl):
	"""Convert an AccessControlLists object into a text record"""
	return '# file: %s\n%s\n' % (C.acl_quote(acl.get_indexpath()), str(acl))

def Record2ACL(record):
	"""Convert text record to an AccessControlLists object"""
	newline_pos = record.find('\n')
	first_line = record[:newline_pos]
	if not first_line.startswith('# file: '):
		raise metadata.ParsingError("Bad record beginning: "+ first_line)
	filename = first_line[8:]
	if filename == '.': index = ()
	else: index = tuple(C.acl_unquote(filename).split('/'))
	return AccessControlLists(index, record[newline_pos:])

class ACLExtractor(EAExtractor):
	"""Iterate AccessControlLists objects from the ACL information file

	Except for the record_to_object method, we can reuse everything in
	the EAExtractor class because the file formats are so similar.

	"""
	record_to_object = staticmethod(Record2ACL)

class AccessControlListFile(metadata.FlatFile):
	"""Store/retrieve ACLs from extended attributes file"""
	_prefix = 'access_control_lists'
	_extractor = ACLExtractor
	_object_to_record = staticmethod(ACL2Record)

def join_acl_iter(rorp_iter, acl_iter):
	"""Update a rorp iter by adding the information from acl_iter"""
	for rorp, acl in rorpiter.CollateIterators(rorp_iter, acl_iter):
		assert rorp, "Missing rorp for index %s" % (acl.index,)
		if not acl: acl = AccessControlLists(rorp.index)
		rorp.set_acl(acl)
		yield rorp
	

def rpath_acl_get(rp):
	"""Get acls of given rpath rp.

	This overrides a function in the rpath module.

	"""
	acl = AccessControlLists(rp.index)
	if not rp.issym(): acl.read_from_rp(rp)
	return acl
rpath.acl_get = rpath_acl_get

def rpath_get_blank_acl(index):
	"""Get a blank AccessControlLists object (override rpath function)"""
	return AccessControlLists(index)
rpath.get_blank_acl = rpath_get_blank_acl

def rpath_ea_get(rp):
	"""Get extended attributes of given rpath

	This overrides a function in the rpath module.

	"""
	ea = ExtendedAttributes(rp.index)
	if not rp.issock() and not rp.isfifo():
		ea.read_from_rp(rp)
	return ea
rpath.ea_get = rpath_ea_get

def rpath_get_blank_ea(index):
	"""Get a blank ExtendedAttributes object (override rpath function)"""
	return ExtendedAttributes(index)
rpath.get_blank_ea = rpath_get_blank_ea




#!/usr/bin/env python

import sys, os, getopt
from distutils.core import setup, Extension

version_string = "1.2.8"

if sys.version_info[:2] < (2,2):
	print "Sorry, rdiff-backup requires version 2.2 or later of python"
	sys.exit(1)

# Defaults
lflags_arg = []
libname = ['rsync']
incdir_list = libdir_list = None
extra_options = {}

if os.name == 'posix' or os.name == 'nt':
	LIBRSYNC_DIR = os.environ.get('LIBRSYNC_DIR', '')
	LFLAGS = os.environ.get('LFLAGS', [])
	LIBS = os.environ.get('LIBS', [])

	# Handle --librsync-dir=[PATH] and --lflags=[FLAGS]
	args = sys.argv[:]
	for arg in args:
		if arg.startswith('--librsync-dir='):
			LIBRSYNC_DIR = arg.split('=')[1]
			sys.argv.remove(arg)
		elif arg.startswith('--lflags='):
			LFLAGS = arg.split('=')[1].split()
			sys.argv.remove(arg)
		elif arg.startswith('--libs='):
			LIBS = arg.split('=')[1].split()
			sys.argv.remove(arg)

		if LFLAGS or LIBS:
			lflags_arg = LFLAGS + LIBS

		if LIBRSYNC_DIR:
			incdir_list = [os.path.join(LIBRSYNC_DIR, 'include')]
			libdir_list = [os.path.join(LIBRSYNC_DIR, 'lib')]
		if '-lrsync' in LIBS:
			libname = []

if os.name == 'nt':
	try:
		import py2exe
	except ImportError:
		pass
	else:
		extra_options = {
			'console': ['rdiff-backup'],
		}
		if '--single-file' in sys.argv[1:]:
			sys.argv.remove('--single-file')
			extra_options.update({
				'options': {'py2exe': {'bundle_files': 1}},
				'zipfile': None
			})

setup(name="rdiff-backup",
	  version=version_string,
	  description="Local/remote mirroring+incremental backup",
	  author="Ben Escoto",
	  author_email="rdiff-backup@emerose.org",
	  url="http://rdiff-backup.nongnu.org/",
	  packages = ['rdiff_backup'],
	  ext_modules = [Extension("rdiff_backup.C", ["cmodule.c"]),
					 Extension("rdiff_backup._librsync",
							   ["_librsyncmodule.c"],
							   include_dirs=incdir_list,
							   library_dirs=libdir_list,
							   libraries=libname,
							   extra_link_args=lflags_arg)],
	  scripts = ['rdiff-backup', 'rdiff-backup-statistics'],
	  data_files = [('share/man/man1', ['rdiff-backup.1',
										'rdiff-backup-statistics.1']),
					('share/doc/rdiff-backup-%s' % (version_string,),
					 ['CHANGELOG', 'COPYING', 'README', 'FAQ.html'])],
					**extra_options)




# Copyright 2002, 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Code for reverting the rdiff-backup directory to prev state

This module is used after an aborted session, and the rdiff-backup
destination directory may be in-between states.  In this situation we
need to bring back the directory as it was after the last successful
backup.  The basic strategy is to restore all the attributes from the
metadata file (which we assume is intact) and delete the extra
increments.  For regular files we examine the mirror file and use the
increment file to get the old data if the mirror file is out of date.

Currently this does not recover hard links.  This may make the
regressed directory take up more disk space, but hard links can still
be recovered.

"""

from __future__ import generators
import signal, errno, re, os
import Globals, restore, log, rorpiter, TempFile, metadata, rpath, C, \
	   Time, backup, robust, longname

# regress_time should be set to the time we want to regress back to
# (usually the time of the last successful backup)
regress_time = None

# This should be set to the latest unsuccessful backup time
unsuccessful_backup_time = None


class RegressException(Exception):
	"""Raised on any exception in regress process"""
	pass


def Regress(mirror_rp):
	"""Bring mirror and inc directory back to regress_to_time

	Also affects the rdiff-backup-data directory, so Globals.rbdir
	should be set.  Regress should only work one step at a time
	(i.e. don't "regress" through two separate backup sets.  This
	function should be run locally to the rdiff-backup-data directory.

	"""
	inc_rpath = Globals.rbdir.append_path("increments")
	assert mirror_rp.index == () and inc_rpath.index == ()
	assert mirror_rp.isdir() and inc_rpath.isdir()
	assert mirror_rp.conn is inc_rpath.conn is Globals.local_connection
	manager, former_current_mirror_rp = set_regress_time()
	set_restore_times()
	regress_rbdir(manager)
	ITR = rorpiter.IterTreeReducer(RegressITRB, [])
	for rf in iterate_meta_rfs(mirror_rp, inc_rpath): ITR(rf.index, rf)
	ITR.Finish()
	if former_current_mirror_rp:
		C.sync() # Sync first, since we are marking dest dir as good now
		former_current_mirror_rp.delete()

def set_regress_time():
	"""Set global regress_time to previous sucessful backup

	If there are two current_mirror increments, then the last one
	corresponds to a backup session that failed.

	"""
	global regress_time, unsuccessful_backup_time
	manager = metadata.SetManager()
	curmir_incs = manager.sorted_prefix_inclist('current_mirror')
	assert len(curmir_incs) == 2, \
		   "Found %s current_mirror flags, expected 2" % len(curmir_incs)
	mirror_rp_to_delete = curmir_incs[0]
	regress_time = curmir_incs[1].getinctime()
	unsuccessful_backup_time = mirror_rp_to_delete.getinctime()
	log.Log("Regressing to " + Time.timetopretty(regress_time), 4)
	return manager, mirror_rp_to_delete

def set_restore_times():
	"""Set _rest_time and _mirror_time in the restore module

	_rest_time (restore time) corresponds to the last successful
	backup time.  _mirror_time is the unsuccessful backup time.

	"""
	restore.MirrorStruct._mirror_time = unsuccessful_backup_time
	restore.MirrorStruct._rest_time = regress_time

def regress_rbdir(meta_manager):
	"""Delete the increments in the rdiff-backup-data directory

	Returns the former current mirror rp so we can delete it later.
	All of the other rp's should be deleted before the actual regress,
	to clear up disk space the rest of the procedure may need.

	Also, in case the previous session failed while diffing the
	metadata file, either recreate the mirror_metadata snapshot, or
	delete the extra regress_time diff.

	"""
	has_meta_diff, has_meta_snap = 0, 0
	for old_rp in meta_manager.timerpmap[regress_time]:
		if old_rp.getincbase_str() == 'mirror_metadata':
			if old_rp.getinctype() == 'snapshot': has_meta_snap = 1
			else:
				assert old_rp.getinctype() == 'diff', old_rp
				has_meta_diff = 1
	if has_meta_diff and not has_meta_snap: recreate_meta(meta_manager)

	for new_rp in meta_manager.timerpmap[unsuccessful_backup_time]:
		if new_rp.getincbase_str() != 'current_mirror':
			log.Log("Deleting old diff at " + new_rp.path, 5)
			new_rp.delete()
	for rp in meta_manager.timerpmap[regress_time]:
		if (rp.getincbase_str() == 'mirror_metadata' and
			rp.getinctype() == 'diff'):
			rp.delete()
			break

def recreate_meta(meta_manager):
	"""Make regress_time mirror_metadata snapshot by patching

	We write to a tempfile first.  Otherwise, in case of a crash, it
	would seem we would have an intact snapshot and partial diff, not
	the reverse.

	"""
	temprp = [TempFile.new_in_dir(Globals.rbdir)]
	def callback(rp): temprp[0] = rp
	writer = metadata.MetadataFile(temprp[0], 'w', check_path = 0, callback = callback)
	for rorp in meta_manager.get_meta_at_time(regress_time, None):
		writer.write_object(rorp)
	writer.close()

	finalrp = Globals.rbdir.append("mirror_metadata.%s.snapshot.gz" %
								   Time.timetostring(regress_time))
	assert not finalrp.lstat(), finalrp
	rpath.rename(temprp[0], finalrp)
	if Globals.fsync_directories: Globals.rbdir.fsync()

def iterate_raw_rfs(mirror_rp, inc_rp):
	"""Iterate all RegressFile objects in mirror/inc directory

	Also changes permissions of unreadable files.  We don't have to
	change them back later because regress will do that for us.

	"""
	root_rf = RegressFile(mirror_rp, inc_rp, restore.get_inclist(inc_rp))
	def helper(rf):
		mirror_rp = rf.mirror_rp
		if Globals.process_uid != 0:
			if mirror_rp.isreg() and not mirror_rp.readable():
				mirror_rp.chmod(0400 | mirror_rp.getperms())
			elif mirror_rp.isdir() and not mirror_rp.hasfullperms():
				mirror_rp.chmod(0700 | mirror_rp.getperms())
		yield rf
		if rf.mirror_rp.isdir() or rf.inc_rp.isdir():
			for sub_rf in rf.yield_sub_rfs():
				for sub_sub_rf in helper(sub_rf):
					yield sub_sub_rf
	return helper(root_rf)

def yield_metadata():
	"""Iterate rorps from metadata file, if any are available"""
	metadata.SetManager()
	metadata_iter = metadata.ManagerObj.GetAtTime(regress_time)
	if metadata_iter: return metadata_iter
	log.Log.FatalError("No metadata for time %s (%s) found,\ncannot regress"
					   % (Time.timetopretty(regress_time), regress_time))

def iterate_meta_rfs(mirror_rp, inc_rp):
	"""Yield RegressFile objects with extra metadata information added

	Each RegressFile will have an extra object variable .metadata_rorp
	which will contain the metadata attributes of the mirror file at
	regress_time.

	"""
	raw_rfs = iterate_raw_rfs(mirror_rp, inc_rp)
	collated = rorpiter.Collate2Iters(raw_rfs, yield_metadata())
	for raw_rf, metadata_rorp in collated:
		raw_rf = longname.update_regressfile(raw_rf, metadata_rorp, mirror_rp)
		if not raw_rf:
			log.Log("Warning, metadata file has entry for %s,\n"
					"but there are no associated files." %
					(metadata_rorp.get_indexpath(),), 2)
			continue
		raw_rf.set_metadata_rorp(metadata_rorp)
		yield raw_rf


class RegressFile(restore.RestoreFile):
	"""Like RestoreFile but with metadata

	Hold mirror_rp and related incs, but also put metadata info for
	the mirror file at regress time in self.metadata_rorp.
	self.metadata_rorp is not set in this class.

	"""
	def __init__(self, mirror_rp, inc_rp, inc_list):
		restore.RestoreFile.__init__(self, mirror_rp, inc_rp, inc_list)
		self.set_regress_inc()
		
	def set_metadata_rorp(self, metadata_rorp):
		"""Set self.metadata_rorp, creating empty if given None"""
		if metadata_rorp: self.metadata_rorp = metadata_rorp
		else: self.metadata_rorp = rpath.RORPath(self.index)

	def isdir(self):
		"""Return true if regress needs before/after processing"""
		return ((self.metadata_rorp and self.metadata_rorp.isdir()) or
				(self.mirror_rp and self.mirror_rp.isdir()))

	def set_regress_inc(self):
		"""Set self.regress_inc to increment to be removed (or None)"""
		newer_incs = self.get_newer_incs()
		assert len(newer_incs) <= 1, "Too many recent increments"
		if newer_incs: self.regress_inc = newer_incs[0] # first is mirror_rp
		else: self.regress_inc = None


class RegressITRB(rorpiter.ITRBranch):
	"""Turn back state of dest directory (use with IterTreeReducer)

	The arguments to the ITR will be RegressFiles.  There are two main
	assumptions this procedure makes (besides those mentioned above):

	1.  The mirror_rp and the metadata_rorp equal_loose correctly iff
	    they contain the same data.  If this is the case, then the inc
	    file is unnecessary and we can delete it.

	2.  If the don't match, then applying the inc file will
	    successfully get us back to the previous state.

	Since the metadata file is required, the two above really only
	matter for regular files.

	"""
	def __init__(self):
		"""Just initialize some variables to None"""
		self.rf = None # will hold RegressFile applying to a directory

	def can_fast_process(self, index, rf):
		"""True if none of the rps is a directory"""
		return not rf.mirror_rp.isdir() and not rf.metadata_rorp.isdir()

	def fast_process(self, index, rf):
		"""Process when nothing is a directory"""
		if not rf.metadata_rorp.equal_loose(rf.mirror_rp):
			log.Log("Regressing file %s" %
					(rf.metadata_rorp.get_indexpath()), 5)
			if rf.metadata_rorp.isreg(): self.restore_orig_regfile(rf)
			else:
				if rf.mirror_rp.lstat(): rf.mirror_rp.delete()
				if rf.metadata_rorp.isspecial():
					robust.check_common_error(None, rpath.copy_with_attribs,
											  (rf.metadata_rorp, rf.mirror_rp))
				else: rpath.copy_with_attribs(rf.metadata_rorp, rf.mirror_rp)
		if rf.regress_inc:
			log.Log("Deleting increment " + rf.regress_inc.path, 5)
			rf.regress_inc.delete()

	def restore_orig_regfile(self, rf):
		"""Restore original regular file

		This is the trickiest case for avoiding information loss,
		because we don't want to delete the increment before the
		mirror is fully written.

		"""
		assert rf.metadata_rorp.isreg()
		if rf.mirror_rp.isreg():
			tf = TempFile.new(rf.mirror_rp)
			tf.write_from_fileobj(rf.get_restore_fp())
			tf.fsync_with_dir() # make sure tf fully written before move
			rpath.copy_attribs(rf.metadata_rorp, tf)
			rpath.rename(tf, rf.mirror_rp) # move is atomic
		else:
			if rf.mirror_rp.lstat(): rf.mirror_rp.delete()
			rf.mirror_rp.write_from_fileobj(rf.get_restore_fp())
			rpath.copy_attribs(rf.metadata_rorp, rf.mirror_rp)
		if Globals.fsync_directories:
			rf.mirror_rp.get_parent_rp().fsync() # force move before inc delete

	def start_process(self, index, rf):
		"""Start processing directory"""
		if rf.metadata_rorp.isdir():
			# make sure mirror is a readable dir
			if not rf.mirror_rp.isdir():
				if rf.mirror_rp.lstat(): rf.mirror_rp.delete()
				rf.mirror_rp.mkdir()
			if not rf.mirror_rp.hasfullperms(): rf.mirror_rp.chmod(0700)
		self.rf = rf

	def end_process(self):
		"""Finish processing a directory"""
		rf = self.rf
		if rf.metadata_rorp.isdir():
			if rf.mirror_rp.isdir():
				rf.mirror_rp.setdata()
				if not rf.metadata_rorp.equal_loose(rf.mirror_rp):
					log.Log("Regressing attributes of " + rf.mirror_rp.path, 5)
					rpath.copy_attribs(rf.metadata_rorp, rf.mirror_rp)
			else:
				rf.mirror_rp.delete()
				log.Log("Regressing file " + rf.mirror_rp.path, 5)
				rpath.copy_with_attribs(rf.metadata_rorp, rf.mirror_rp)
		else: # replacing a dir with some other kind of file
			assert rf.mirror_rp.isdir()
			log.Log("Replacing directory " + rf.mirror_rp.path, 5)
			if rf.metadata_rorp.isreg(): self.restore_orig_regfile(rf)
			else:
				rf.mirror_rp.delete()
				rpath.copy_with_attribs(rf.metadata_rorp, rf.mirror_rp)
		if rf.regress_inc:
			log.Log("Deleting increment " + rf.regress_inc.path, 5)
			rf.regress_inc.delete()


def check_pids(curmir_incs):
	"""Check PIDs in curmir markers to make sure rdiff-backup not running"""
	pid_re = re.compile("^PID\s*([0-9]+)", re.I | re.M)
	def extract_pid(curmir_rp):
		"""Return process ID from a current mirror marker, if any"""
		match = pid_re.search(curmir_rp.get_data())
		if not match: return None
		else: return int(match.group(1))

	def pid_running(pid):
		"""True if we know if process with pid is currently running"""
		try: os.kill(pid, 0)
		except OSError, exc:
			if exc[0] == errno.ESRCH: return 0
			else: log.Log("Warning: unable to check if PID %d still running" % (pid,), 2)
		except AttributeError:
			assert os.name == 'nt'
			import win32api, win32con, pywintypes
			process = None
			try:
				process = win32api.OpenProcess(win32con.PROCESS_ALL_ACCESS, 
											0, pid)
			except pywintypes.error, error:
				if error[0] == 87: return 0
				else:
					msg = "Warning: unable to check if PID %d still running"
					log.Log(msg % pid, 2)
			if process:
				win32api.CloseHandle(process)
				return 1
			return 0
		return 1

	for curmir_rp in curmir_incs:
		assert Globals.local_connection is curmir_rp.conn
		pid = extract_pid(curmir_rp)
		if pid is not None and pid_running(pid):
			log.Log.FatalError(
"""It appears that a previous rdiff-backup session with process
id %d is still running.  If two different rdiff-backup processes write
the same repository simultaneously, data corruption will probably
result.  To proceed with regress anyway, rerun rdiff-backup with the
--force option.""" % (pid,))
